import extend from 'extend'
import Clone from 'core/clone'
import Time from 'time'
import Utils from 'utils'
import Ajaxqueue from 'ajax_queue'
import Layer from 'chart/layer'
import BaseProduct from 'chart/model/base_product'
import Parameters from 'chart/model/parameters'

function Product(args) {
	this.time = args.time
}

Product.prototype = extend(new BaseProduct(), {
	init: function(o) {
		this.state = {
			// assumes string
			product: o.product,
		}

		return this
	},

	get_chart: function(request) {
		var self = this;
		var success = request.success || Utils.noop;

		if(!request.json) request.json = {product: this.as_json()}

		function _need_old_api(product) {
			// note that it is common for projection to be an object, so that cannot be used as a test
			// note that not only eccharts have layers, so that cannot be used as a test

			if(self.kind === 'chart') return true; // temporary fix, kind option is passed from chart dashboard widget
			if(self.kind === 'interactive') return false; // temporary fix, kind option is passed from chart dashboard widget

			if(product.product === 'empty') return true;
			if(product.product.indexOf('uid_') === 0) return true; // uid_ charts are always eccharts charts but not all eccharts charts are uid_ charts.

			return false;
		}

		if(self.api > 1.0){
			var new_request = extend(true, {}, request.json.product); // this variable only used for comparisons.

			delete new_request.time;
			delete new_request.layers;
			delete new_request.availability;
			delete new_request.magics_viewport;
			delete new_request.use_product_projection;
			delete new_request.viewport.top;
			delete new_request.viewport.left;

			if(JSON.stringify(new_request) === JSON.stringify(self.__prev_request)){
				return console.log('product.get_chart: ignoring duplicate request');
			}
			self.__prev_request = new_request;
		}

		if(__DEV__){
			// check dimensions - this is done because if dimensions are invalid,
			// the backend will fail in ways that are very difficult to debug (will timeout)
			if(request.json.product && request.json.product.viewport){
				var viewport = request.json.product.viewport;
				if(viewport.width !== undefined) viewport.width = Math.max(viewport.width, 48)
				if(viewport.height !== undefined) viewport.height = Math.max(viewport.height, 32)
			}
		}

		Ajaxqueue(extend(
			{
				url: _need_old_api(request.json.product) ? '/backend/get-chart/' : ECMWF.options.ajax.backend.get_chart,
				error: Utils.noop
			},
			self._req_defaults,
			request,
			{
				success: function(json) {
					/*
					if(__DEV__){
						var is_eccharts = self.api < 2 && json.result.layers && json.result.layers > 0;
						if(!self.validate([
							['has result', function(){ return $.isPlainObject(json.result); }],
							['has values', function(){ return !is_eccharts || json.values; }],
							['values are valid', function() {
								if(!is_eccharts || json.axis) {
									return !$.map(json.values, function(val, param) {
										var axis = Utils.findWhere(json.axis, {name: param});
										if(!~$.inArray(val, axis.values)) return val;
									}).length;
								}
								return true;
							}],
						])){
							return request.error(self.error_msg);
						}
					} else*/ if (!json.result || !Utils.isPlainObject(json.result)) {
						return request.error.apply(this, arguments);
					}

					success(json, self.___update(json.product, json.result.magics, json.availability, json.axis, json.values, json.widgets));
				}
			}
		))

		return this
	},

	___update: function(result, magics, availability, axes, values, widgets) {
		var self = this, time;

		this.data.product = {widgets: widgets};

		if (result.title) {
			this._title = result.title;
		}

		this.state.projection = result.projection;

		result.layers.sort(by_z_index);

		this.layers_by_id = {};

		result.layers.forEach(function(layer, i) {
			if (layer.time) {
				time = new Time(layer.time);
			}

			layer.id || (layer.id = ++next_id)
			layer.values || (layer.values = {})
			layer.styles || (layer.styles = [])
			layer.variables || (layer.variables = [])

			self.layers_by_id[layer.id] = layer;

			delete layer.time
			delete layer.values.base_time
			delete layer.values.step
			delete layer.values.valid_time
		})

		if (availability) {
			self._updated = availability.updated;
			self._availability = availability.availability.map(function(a) {
				return new Time(a);
			})
			self._update_number = ++BaseProduct.prototype._update_number;
		}

		this.state.layers = result.layers;

		// Store the variables as sent from the server.
		/*
		if(self.api > 1.0){
			if (axes) {
				var variables = availability ? {time: self.variables.time} : {};
				$.each(axes, function(i, v) {
					if(values[v.name]) v.value = self.__prev_request.values[v.name] = values[v.name];

					variables[v.name] = self.variables[v.name]
						? self.variables[v.name].set(v.value, v.values, v.labels)
						: ((v.type === 'dynamic')
							? new Parameters.dynamic(v)
							: new Parameters.string(v));
				})
				self.variables = variables;
			}

			if(availability){
				self.__prev_request.values.time = [result.time.base_time.substr(0, 10), result.time.step, result.time.valid_time.substr(0, 10)];
				self.variables.time.set(new Time(result.time), self._availability);
			}
		}
		*/

		return time;
	},

	layerList: function(filter) {
		filter || (filter = function() { return true; })

		return this.state.layers.filter(function(l) {
			return filter.call(l)
		})
	},

	as_json: function(viewport, time) {
		var self = this;
		time = time || this.time

		var data = Utils.extend(true,
			{},
			this.state,
			{
				viewport: {top: 0, left: 0, width: 128, height: 64},
				format: 'png'
			}
		)

		// TODO: Come back here
		if (!self._availability) {
			data.availability = true;
		}

		self.state = self.state || {layers: []}

		data.layers && data.layers.forEach(function(layer, i) {

			layer.user_title = new Layer(layer).get_title();

			if (!self.state.layers[i].values) {
				self.state.layers[i].values = {};
			}

			delete layer.styles; // save some bandwidth
			delete layer.units;
			delete layer.title;
			delete layer.variable_title;
			delete layer.variables;
			delete layer.has_probe;
			delete layer.has_data;
		})

		if (time && time.base_time) {
			data.time = time.toJSON();
		}

		data = Clone.cloneObject(data);

		if (!data.values) {
			data.values = {};
		}

		for(var i in self.variables){
			if(self.variables[i].value) data.values[self.variables[i].name] = self.variables[i].for_json();
		}
		time && (data.values.time = new Parameters.time(time).for_json());
		delete data.time;
		//delete data.projection;

		return data
	},

	layer_icon: function(layer_id, img, time, done) {
		Ajaxqueue({
			// TODO: pass the map name
			cancelled_by: this.owner,
			url: '/backend/layer-icon/',
			priority: 'low',
			json: {
				product: this.as_json(),
				layer: layer_id
			},
			img: img,
			success: done || Utils.noop
		})
		return this;
	},

	add_layer: function(layer, done) {
		this.state.layers.push({
			name: layer.name(),
			//TODO these are needed ?
			'z_index': 2000,
			inited: false
		})

		this.get_chart({success: function(){
			done()
		}})
	},

	remove_layer: function(layer_id) {
		var layer = this.layers_by_id[layer_id];
		this.state.layers = this.state.layers.filter(function(a) {
			return a !== layer;
		})
		this._availability = null;
	}
})

var next_id = 0;

function by_z_index(a, b) {
	return a.z_index - b.z_index;
}

export default Product

