/*
 *  We have 'settings' and 'preferences'. Preferences can be explicitly modified
 *  by the user in the Preferences dialogue, but 'settings' are saved in the background.
 */
import ECMWF from 'config'
import Ajaxqueue from 'ajax_queue'
import Utils from 'utils'
import schema from 'core/preferences_schema'

var data;
var enable = ECMWF.options.enable_settings !== false;

function load_data(ready) {
	ready = ready || Utils.noop;

	if (data) {
		ready(data);
		return;
	}

	Ajaxqueue({
		url: '/backend/get-user-data/',
		priority: 'vip',
		success: function(result) {
			data = result;
			ready(data);
		},
		error: function(result) {
			// Ignore for now ....
			data = {};
			ready(data);
		}
	})
}

function store_data(ready) {
	ready = ready || Utils.noop;

	delete data.schema;

	Ajaxqueue({
		url: '/backend/set-user-data/',
		json: data,
		name: 'set_user_data',
		success: function(result) {
			data = result;
			ready();
		}
	})
}

/* TODO
Bus().listen('preferences-changed', function(_, editable_prefs) {
	var old_prefs = data.preferences;
	data.preferences = editable_prefs;
	for (var i in old_prefs) {
		if (typeof data.preferences[i] === 'undefined' && schema[i] && schema[i].editable === false) {
			data.preferences[i] = old_prefs[i];
		}
	}
	store_data();
});
*/

function set_data(kind, name, value) {
	if (!data[kind]) data[kind] = {};
	var old = data[kind][name];
	data[kind][name] = value;
	store_data();
	return old;
}

var warning_done;

function get_data(kind, name, value) {
	if (!data) data = {};
	if (!data[kind]) data[kind] = {};
	if (typeof data[kind][name] === 'undefined') {
		data[kind][name] = value;
		if (typeof data[kind][name] === 'undefined') {
			//set default value from schema
			if (typeof schema === 'undefined') {
				warning_done || (console.warn('schema=', schema), warning_done = true);
				return null;
			}
			var s = schema[name];
			return s ? s.value : null;
		}
	}
	return data[kind][name];
}

var Preferences = {
	set: function(name, value) {
		return set_data('preferences', name, value);
	},

	get: function(name, defaults) {
		return get_data('preferences', name, defaults);
	}
}

var Settings = {
	schema: schema,

	data: new Promise(function(resolve) {
		if(enable){
			load_data(resolve)
		}else{
			resolve()
		}
	}),

	get: function(name, defaults) {
		return get_data('settings', name, defaults);
	},

	set: function(name, value) {
		// name           : top-level key within 'settings'. If not set, value is assumed to be the whole settings data.
		// value          : the data to set

		if(!enable) return;

		var s = {};
		if (name) {
			s[name] = value;
		} else {
			s = value;
		}

		data.settings = data.settings || {};
		if (name) data.settings[name] = value;
		else data.settings = value;

		Ajaxqueue({
			url: '/backend/set-user-setting/',
			json: s
		});
	},

	get_pref: function(name, defaults) {
		return get_data('preferences', name, defaults);
	},

	each_pref: function(callback) {
		data.preferences = data.preferences || {};
		for (var i in schema) {
			var pref = schema[i];
			if (typeof pref === 'object') {
				pref.value = (typeof data.preferences[pref.name] !== 'undefined') ? data.preferences[pref.name] : pref.value;
				callback(pref);
			}
		}
	}
}

//export {Settings, Preferences} // change to this later
export default {Settings: Settings, Preferences: Preferences}
