import h from 'virtual-dom/h'
import mixins from 'mixins'
import Button from 'grid/button'
import ToggleButton from 'grid/togglebutton'
import Slider from './canvas_slider'
import extend from 'extend'
import 'time-navigator/time_navigator.css'

function TimeNavigator(args) {
	var self = this

	var o = this.options = extend({
		time: null,     // type Observable
		animator: null, // type ChartAnimator
	}, args);

	this.buttons = [[
		new Button({
			icon: 'ui-icon-carat-1-w',
			className: 'prev-step',
			on_click: function() {
				console.log('prev.click')
				o.time.previous()
			}
		}),
		new Button({
			icon: 'ui-icon-carat-1-e',
			className: 'next-step',
			on_click: function() {
				console.log('next.click')
				o.time.next()
			}
		})],
		[
		new ToggleButton({
			icons: ['ui-icon-play', 'ui-icon-pause'],
			className: 'animate',
			on: function() {
				console.log('animate start')
				o.animator.toggle()
			},
			off: function() {
				console.log('animate stop')
				o.animator.toggle()
			},
			renderer: o.renderer
		})
		]
	]

	this.slider = new Slider({
		height: 24,
		renderer: o.renderer,
		on_select: this.bind(this._send_time_change_from_date),
		range_selection: {
			enabled: true,
			on_select: this.bind(self._send_animation_selection_change)
		}
	})

	o.availability.subscribe(this.bind(this._update))

	o.time.subscribe(function(time) {
		self.slider.setVal(time.valid_time)
		self.current_idx = time.index(self.dates)
		self.update_button_states()
	})
}

TimeNavigator.prototype = extend({
	render: function() {
		return h('.time-navigator', [
			h('.time-nav', [
				h('.buttons', this.buttons[0].map(function(b){ return b.render() })),
				h('span', 'VT:'),
				this.slider.render(),
				this.buttons[1][0].render()
			])
		])
	},

	resize: function(width) {
		this.slider.size(width)
	},

	_update: function(msg) {
		var self = this, o = self.options;

		var valid_times = o.time.valid_times()

		this.dates       = valid_times.values;
		this.current_idx = valid_times.index;
		this.ref_date    = valid_times.values[0].base_time;

		this.primary = {
			axis: msg.valid_time
		}

		this.last_msg = {
			valid_times: valid_times
		}

		this._set_selectlist();
		/* TODO
		(this.secondary.dates.length && o.show_bt) || (this.selector.hide(), this.secondary_label.hide());
		*/

		if(self.dates.length > 1)
			this._enable();
		else
			this._disable();

		self.slider.setDates({
			dates: (self.dates)
				? self.dates.map(function(a) { return a.valid_time})
				: null,
			index: self.current_idx,
			ref: self.ref_date,
			format: msg.format
		})
	},

	_enable: function() {
	},

	_disable: function() {
	},

	update_button_states: function(){
		var length = this.last_msg.valid_times.values.length;

		this.buttons[0][0].disabled = this.current_idx === 0
		this.buttons[0][1].disabled = this.current_idx === length - 1

		this.options.renderer.render()
	},

	_set_selectlist: function() {
		if(this.primary && this.primary.axis){
			var primary_dates = this.primary.axis.values.map(function(a) { return a.valid_time });
			//var secondary_dates = this.secondary.dates.map(function(a) { return a[self.secondary.axis_name] });
			var not_latest = (this.primary.axis.index != primary_dates.length - 1)

			/* TODO
			this.selector.dateSelect('setDates', secondary_dates, secondary_dates[this.secondary.index], (false && not_latest && !this.flipped));

			self.selector.find('.button div').pulsate(not_latest);
			*/

			this.is_latest = !not_latest;
		}
	},

	_send_time_change_from_date: function(date){
		this.options.time.set(date)
	},

	_send_animation_selection_change: function(selection){
	},
}, mixins)

export default TimeNavigator
